
/* pvmatch -- match vdata buffer to structure fields
 *
 * 
 * An FLIST is an array of records describing fields in a buffer.
 * The records contain field name, HDF data type, and the vdata
 * field "order", i.e., the number of elements in a field.
 * 
 * For each element in the f1 list, pvmatch tries to find a match
 * in the f2 list.  It returns two vectors of indices, p1 and p2;
 * p1 is field offsets in the f1 buffer, and p2 is offsets to to
 * the corresponding fields in the f2 buffer; p3 gives the number
 * of elements to copy.  
 *
 * flist 1 would normally be the vdata buffer, and flist 2 the 
 * structure buffer used to pass data to and from the vdata
 *
 * Here is an example with two disjoint flists, with index arrays 
 * p1, p2 and p3 shown after a call to pvmatch
 * 
 *            flist 1                    flist 2       
 *    name     type    order     name     type    order
 *    ------   -----   -----     ------   -----   -----
 *    fish     INT32     2       fish     INT32     2  
 *    berry    INT32     1       monkey   INT32     3  
 *    turnip   FLOAT32   4       turnip   FLOAT32   4  
 *    monkey   FLOAT32   4       cheese   INT32     3
 *                               beer     FLOAT32   4
 *     p1    p2    p3    
 *    ----  ----  ----   * p1 is the start of the field in the   
 *      0     0     2      flist 1 buffer (the vdata buffer)
 *      2    BAD    0    * p2 is the start of the matching field
 *      3     5     4      in flist 2 buffer (the structure buffer)
 *      7     2     4    * p3 is the number of values to copy
 *
 * 
 * H. Motteler
 * 12 Jan 01
 *
 */

#include <stdlib.h>
#include "rtp.h"

void pvmatch(
	    struct FLIST (*f1)[],  /* vdata field list		   IN  */
	    int n1,		   /* number of vdata fields 	   IN  */
	    struct FLIST (*f2)[],  /* structure field list 	   IN  */
	    int n2,		   /* number of structure fields   IN  */
	    int **pp1,		   /* positions in vdata buffer    OUT */
	    int **pp2,		   /* positions in structure buf   OUT */
	    int **pp3,		   /* number of elements to copy   OUT */
	    int *s1,		   /* size of vdata buffer	   OUT */
	    int *s2		   /* size of structure buf	   OUT */
	    ) {

  int i, j, k;
  int *p1, *p2, *p3, *p4;

  /* get space for returned and working arrays */
  *pp1 = p1 = (int *) malloc(n1 * sizeof(int));
  *pp2 = p2 = (int *) malloc(n1 * sizeof(int));
  *pp3 = p3 = (int *) malloc(n1 * sizeof(int));
  p4 = (int *) malloc(n2 * sizeof(int));


  /* get offsets in FLIST 1, the vdata buffer */
  for (i=0,j=0; i < n1; i++) {
    p1[i] = j;			/* offset from start of the record */
    p3[i] = (*f1)[i].order;	/* copy length of field to p3 */
    j += p3[i];
  }
  *s1 = j;

  /* get offsets in FLIST 2, the struct buffer */
  for (i=0,j=0; i < n2; i++) {
    p4[i] = j;
    j += (*f2)[i].order;
  }
  *s2 = j;

  /* search for matching fields, set p2 on a match and zero p3 
     if no match is found; k flags a match */
  for (i=0; i < n1; i++) {
    for (j=0,k=0; j < n2; j++)
      if (strcmp((*f1)[i].fname, (*f2)[j].fname)==0) {
	p2[i] = p4[j];
	k = 1;
      }
    if (k == 0) {
      p3[i] = 0;
      p2[i] = BAD;
    }
  }
}

